@@ -43,7 +43,14 @@ def check_recursive_and_fname(args):
 
 
 def transform_targets(args):
-    args.targets = ensure_list(args.targets)
+    from funcy import count_reps
+
+    counts = count_reps(ensure_list(args.targets))
+    dupes = [key for key, count in counts.items() if count > 1]
+    if dupes:
+        msg = ", ".join(f"[b]{key}[/]" for key in dupes)
+        ui.error_write(f"ignoring duplicated targets: {msg}", styled=True)
+    args.targets = list(counts)
 
 
 def check_arg_combinations(args):
@@ -162,19 +169,17 @@ def add(  # noqa: C901
     desc = "Collecting targets"
     stages_it = create_stages(repo, add_targets, fname, transfer, **kwargs)
     stages = list(ui.progress(stages_it, desc=desc, unit="file"))
-
     msg = "Collecting stages from the workspace"
     with translate_graph_error(stages), ui.status(msg) as status:
         # remove existing stages that are to-be replaced with these
         # new stages for the graph checks.
-        old_stages = set(repo.stages) - set(stages)
+        new_index = repo.index.update(stages)
         status.update("Checking graph")
-        repo.check_modified_graph(stages, list(old_stages))
+        new_index.check_graph()
 
     odb = None
     if to_remote:
-        remote = repo.cloud.get_remote(kwargs.get("remote"), "add")
-        odb = remote.odb
+        odb = repo.cloud.get_remote_odb(kwargs.get("remote"), "add")
 
     with warn_link_failures() as link_failures:
         for stage, source in zip(progress_iter(stages), sources):
