@@ -1,18 +1,24 @@
-import sys
+from contextlib import contextmanager
 from typing import (
     TYPE_CHECKING,
     Any,
+    Callable,
     Dict,
     Iterable,
+    Iterator,
     Optional,
     Sequence,
     TextIO,
+    Type,
     Union,
 )
 
 from funcy import cached_property
 
 if TYPE_CHECKING:
+    from rich.status import Status
+    from rich.text import Text as RichText
+
     from dvc.progress import Tqdm
     from dvc.ui.table import Headers, Styles, TableData
 
@@ -38,8 +44,13 @@ class Console:
     def __init__(
         self, formatter: Formatter = None, enable: bool = False
     ) -> None:
+        from contextvars import ContextVar
+
         self.formatter: Formatter = formatter or Formatter()
         self._enabled: bool = enable
+        self._paginate: ContextVar[bool] = ContextVar(
+            "_paginate", default=False
+        )
 
     def enable(self) -> None:
         self._enabled = True
@@ -59,6 +70,8 @@ class Console:
         style: str = None,
         sep: str = None,
         end: str = None,
+        styled: bool = False,
+        force: bool = True,
     ) -> None:
         return self.write(
             *objects,
@@ -66,8 +79,27 @@ class Console:
             sep=sep,
             end=end,
             stderr=True,
+            force=force,
+            styled=styled,
         )
 
+    def write_json(
+        self, data: Any, indent: int = 2, default: Callable = None
+    ) -> None:
+        import json
+
+        if self.isatty():
+            from rich.highlighter import JSONHighlighter
+
+            j = json.dumps(data, indent=indent, default=default)
+            highlighter = JSONHighlighter()
+            text = highlighter(j)
+            text.no_wrap = True
+            text.overflow = None
+            return self.write(text, styled=True)
+
+        return self.write(json.dumps(data))
+
     def write(
         self,
         *objects: Any,
@@ -79,18 +111,32 @@ class Console:
         styled: bool = False,
         file: TextIO = None,
     ) -> None:
+        import sys
+
+        from dvc.progress import Tqdm
+
         sep = " " if sep is None else sep
         end = "\n" if end is None else end
         if not self._enabled and not force:
             return
 
-        if styled:
-            console = self.error_console if stderr else self.rich_console
-            return console.print(*objects, sep=sep, end=end)
-
         file = file or (sys.stderr if stderr else sys.stdout)
-        values = (self.formatter.format(obj, style=style) for obj in objects)
-        return print(*values, sep=sep, end=end, file=file)
+        with Tqdm.external_write_mode(file=file):
+            # if we are inside pager context, send the output to rich's buffer
+            if styled or self._paginate.get():
+                console = self.error_console if stderr else self.rich_console
+                if styled:
+                    return console.print(*objects, sep=sep, end=end)
+                return console.out(*objects, sep=sep, end=end, highlight=False)
+
+            values = (self.formatter.format(obj, style) for obj in objects)
+            return print(*values, sep=sep, end=end, file=file)
+
+    @property
+    def rich_text(self) -> "Type[RichText]":
+        from rich.text import Text
+
+        return Text
 
     @staticmethod
     def progress(*args, **kwargs) -> "Tqdm":
@@ -98,6 +144,17 @@ class Console:
 
         return Tqdm(*args, **kwargs)
 
+    @contextmanager
+    def pager(self, styles: bool = True) -> Iterator[None]:
+        from .pager import DvcPager
+
+        tok = self._paginate.set(True)
+        try:
+            with self.rich_console.pager(pager=DvcPager(), styles=styles):
+                yield
+        finally:
+            self._paginate.reset(tok)
+
     def prompt(
         self, text: str, choices: Iterable[str] = None, password: bool = False
     ) -> Optional[str]:
@@ -181,6 +238,14 @@ class Console:
             self, data, headers, markdown=markdown, pager=pager, force=force
         )
 
+    def status(self, status: str, **kwargs: Any) -> "Status":
+        return self.error_console.status(status, **kwargs)
+
+    def isatty(self) -> bool:
+        import sys
+
+        return sys.stdout.isatty()
+
 
 ui = Console()
 
